<![CDATA[

	// encapsulation of the code that looks up a data type
	function CDType()
	{	
		this._bInitialized = false;
	}

	CDType.prototype.SetErrorAndFail = function(sErr)
	{
		this._error = sErr;
		this.ResetInit();
		return false;
	}

	CDType.prototype.ResetInit = function()
	{
		this._bInitialized = false;
	}

	CDType.prototype.Init = function()
	{
		if (this._bInitialized)
		{
			return;
		}
		
		delete this._error;
		delete this._oDTRef;
		delete this._sType;
		delete this._sRID;
		delete this._sDLang;
		this._sCSSClass = "clsUnspecified";

		this._bInitialized = true;

		return true;
	}

	// Ensure that the lookup table is loaded
	CDType.prototype.EnsureDTypes = function(oCtx)
	{
		if (!this._data)
		{
			this._data = new ActiveXObject("MSXML.DOMDocument");
			this._data.async = false;
			this._data.load(oCtx.GetPathOf("dtypes"));
			var oPErr = this._data.parseError;
			if (oPErr.errorCode)
			{
				return this.SetErrorAndFail(oPErr.URL + ": " + oPErr.reason);
			}
		}

		return true;
	}

	// The bNoInit flag is used to prevent this routine from wiping out the 
	CDType.prototype.LookupByID = function(sRID, oCtx)
	{
		if (!this.EnsureDTypes(oCtx))
		{
			return this.SetErrorAndFail(this._error);
		}

		// CONSIDER: Optimization would be to examine the last lookup
		// CAUTION: VARIANT case is tricky since vtype combos may differ; also consider author-specified @devlang on dtype
		this.Init();

		if (!this._sDLang)
		{
			this._sDLang = oCtx._devlang;
		}

		if (!this._sDLang)
		{
			return this.SetErrorAndFail("devlang not specified globally or on dtype.");
		}

		this._sRID = sRID;

		switch (this._sRID)
		{
		case "VARIANT":			
			var sType = this.HandleVariant();
			if (!sType)
			{
				return this.SetErrorAndFail(this._error);
			}
			this._sType = sType;
			break;
		default:
			var sType = this.LookupType(this._sRID);
			if (null == sType)
			{
				return this.SetErrorAndFail(this._error);
			}
			this._sType = sType;			
		}

		this.ResetInit();

		return true;
	
	}

	CDType.prototype.Lookup = function(oDTRef, oCtx, sKey)
	{
		if (!sKey)
		{
			sKey = "rid";
		}

		var sRID = oDTRef.getAttribute(sKey);

		if (!sRID)
		{
			return this.SetErrorAndFail(sKey + " not specified on dtype");
		}

		this.Init();

		this._oDTRef = oDTRef;

		this._sDLang = oDTRef.getAttribute("devlang");
		
		return this.LookupByID(sRID, oCtx);
	}

	CDType.prototype.HandleVariant = function()
	{
		var s = this.LookupType("VARIANT"); // formatting may vary depending on devlang
		if (!s)
		{
			return null;
		}

		// LookupByID must have been called directly from a template, so a node is not cached
		// the schema doesn't support vtype under a param
		if (!this._oDTRef)
		{
			return s;
		}		
		
		var oVTColl = this._oDTRef.selectNodes("vtype");

		var iSubTypes = oVTColl.length;		
		if (iSubTypes == 0)
		{
			return s;
		}

		if (this._sDLang == "scr")
		{
			s = "";
		}
		else
		{
			s += " of type ";
		}
		
		switch (iSubTypes)
		{
		
		case 0:
			break;
		case 1:
			var sType = this.LookupType(oVTColl(0).getAttribute("rid"));
			if (null == sType)
			{
				return null;
			}
			s += sType;
			break;
		case 2:
			var sType1 = this.LookupType(oVTColl(0).getAttribute("rid"))
			if (null == sType1)
			{
				return null;
			}
			var sType2 = this.LookupType(oVTColl(1).getAttribute("rid"))
			if (null == sType2)
			{
				return null;
			}
			s += sType1 + " or " + sType2;
			break;
		default:
			for (var i = 0; i < iSubTypes; i++)
			{
				var sType = this.LookupType(oVTColl(i).getAttribute("rid"))
				if (null == sType)
				{
					return null;
				}
				s += sType;
				if (iSubTypes-2 == i)
				{
					s += ", or ";
				}
				else if (i < iSubTypes-1)
				{
					s += ", ";
				}
			}
		}

		return s;
		
	}

	CDType.prototype.LookupType = function(sRID)
	{
		var sQuery = "id('" + sRID + "')/langdisp[@name='" + this._sDLang + "']/@type";
		var oTypeAttr = this._data.selectSingleNode(sQuery);
		if (!oTypeAttr)
		{
			this._error = "type '" + sRID + "' not found for devlang '" + this._sDLang + "'";
			return null;
		}
		
		sQuery = "id('" + sRID + "')[@unformatted]";
		var oUnformattedAttr = this._data.selectSingleNode( sQuery );
		if ( oUnformattedAttr )
		{
			return oTypeAttr.value;
		}
		else
		{
			return "<b>" + oTypeAttr.value + "</b>";
		}
	}

	CDType.prototype.GetType = function()
	{
		var oDTRef = this._oDTRef;
		if (oDTRef && oDTRef.getAttribute("initcap"))
		{
			return Proper(this._sType);
		}
		else
		{
			return this._sType;
		}
	}

	CDType.prototype.GetTypeNoTags = function()
	{
		var sString = this.GetType();
		var iStart = 0;
		var iEnd = 0;
		
		while ((iStart = sString.indexOf("<")) != -1)
		{
			iEnd = sString.indexOf(">", iStart);
			sString = sString.substring(0, iStart) + sString.substring(iEnd + 1);
		}

		return sString;
	}

	CDType.prototype.GetClass = function()
	{
		return this._sCSSClass;
	}

	// return the last error that occurred during image lookup
	CDType.prototype.GetError = function()
	{
		return (typeof(this._error) != 'undefined' ? this._error : '');
	}

]]>
