
//Creates an isolation layer between the document and the method of tranformation.
//Allows for:
//				The DOM can be manipulated before a trnasform is applied. Example: Insert member data into the page.	
//				A series of transforms can be applied. Example: Transform content, transform xrefs, transform TLAs.
//				Mini transforms can be run on pieces of the document. Example: Just transform the description.
//				Allows the XSL to be cached via LUCache.
//				Allows for any COM based processing such as regular expression SRs, logging, server event notification. Example: Update the index file, update the targets file, strip comments, find "bad words" and write to a log.
//				Allows a document to be constructed from pieces that may be stored seperately. Example: Document may contain description, but parameters may be in source code.
//				Allows data pages to be built via XSL. Example: Instead of "build scripts", just have a set of data files that are built via a transform.

var gxmlDoc = new ActiveXObject("Microsoft.XMLDOM");	//The current document reloaded so it can be written to
var gxslDoc = new ActiveXObject("Microsoft.XMLDOM");	//The stylesheet normally used to to trnasform this document
var gErrorMessage;

//main is called from the XSL. It returns the transformed document.
function main(oNode)
{
//debugger;
	var sResult="Unknown Failure";
	gxmlDoc = oNode.cloneNode(true);
	if (!gxmlDoc || gxmlDoc.xml == "")
	{
		sResult = "Unable to clone XML doc node";
		return sResult;
	}
//	sDocPath=getPath(oNode.url);		//The path to this document
//	if (!reload(sDocPath))
//	{
//		sResult=gErrorMessage;
//		return sResult;
//	}
	if (!manipulateDOM())				//Any changes made to the document are done here
	{
		sResult=gErrorMessage;
		return sResult;
	}
	sResult=transformDoc();				//Now transform the doc and return the results to the XSL
	return sResult;
}

//reloadDocument function reloads this document into a global dom variable. Returns success or failure.
function reload(sPath)
{
	var bResult=true
	gxmlDoc.validateOnParse=false
	gxmlDoc.async=false
	gxmlDoc.resolveExternals=true
	gxmlDoc.load(sPath)
	var parseErr=gxmlDoc.parseError
	if (parseErr.errorCode != 0)
	{
		gErrorMessage= sPath + " " + parseErr.reason + " " + parseErr.line + " " + parseErr.srcText;
		bResult=false;
	}
	return bResult;
}

//getPath strips the file protocol from the document URL if it exists
function getPath(sPath)
{
	var sResult=sPath	//default is do nothing
	var length=sPath.length;
	if (sPath.indexOf('file:///')!=(-1))
	{
		sResult= sPath.substr(8,length-8);
	}
	else
	{
		if (sPath.indexOf('file://')!=(-1))
		{
			sResult= sPath.substr(7,length-7);
		}
	}
	return sResult;
}

//manipulateDOM alters the original document;
function manipulateDOM()
{
	var sType = gxmlDoc.selectSingleNode("/inetsdk:topic/metadata/@type").text;
	if (sType != "event")
		return insertMemberData();
	else
	{
		var oEventProps = gxmlDoc.selectNodes("/inetsdk:topic/content/eventprops/xref");

		for (i = 0; i != oEventProps.length; i++)
		{
			var sID = oEventProps[i].getAttribute("iid");
			if (!sID) sID = "_inet_event_Object"
			var oParNode = gxmlDoc.selectSingleNode("/inetsdk:topic/parent[@id='" + sID + "']");
			if (!oParNode)
				insertMemberData(sID);
		}
		return true;
	}
}

//insertMemberData looks up the member data for this interface
//Specific to interface pages
function insertMemberData(sID)
{
	bNoLuCache = false;
	
	bNoLuCache = "&cache_lookups;"=="1" ? false : true;
	try{
		var luc = new ActiveXObject("LuCache.Lookup");
	} catch( e ) {
		bNoLuCache = true;
	}
	var bResult=true
	if (!sID)
		var docId = gxmlDoc.selectSingleNode("/inetsdk:topic/metadata/@id").text;
	else
		var docId = sID;
	var dataDoc=new ActiveXObject("Microsoft.XMLDOM");
	dataDoc.validateOnParse=true
	dataDoc.async=false
	dataDoc.resolveExternals=true
	sDataPath = "&path_members;"

	if( bNoLuCache ){
		dataDoc.load(sDataPath);
		var parseErr=dataDoc.parseError
		if (parseErr.errorCode != 0)
		{
			gErrorMessage= sDataPath + " " + parseErr.reason + " " + parseErr.line + " " + parseErr.srcText;
			bResult=false;
			return bResult;
		}
		var objectData=dataDoc.nodeFromID(docId);
	} else {
		var objectData = new ActiveXObject("Microsoft.XMLDOM");
		objectData.validateOnParse = false;
		objectData.resolveExternals = true;
		var sCleanPath=getPath(gxmlDoc.documentElement.ownerDocument.url);
		if (sCleanPath.indexOf('http://')!=(-1))
		{
			sCleanPath= "";
		}
		else
		{
			sCleanPath=sCleanPath.substr(0,2);
		}

		try{
			var sNodeRep = "&lt;!DOCTYPE parent SYSTEM '/production/xml/dtd/symbols_v1.dtd' []&gt;" + luc.LookupId( docId, sCleanPath + sDataPath );
			objectData.loadXML(sNodeRep);
			objectData = objectData.documentElement;
		} catch(e) {
			// peterril:  just added safty.
			gErrorMessage = "Unable to query objectmembers for parent node.";
			bResult = false;
			return bResult;
		}
	}
	if (objectData)
	{
		var topicNode=gxmlDoc.selectSingleNode("/inetsdk:topic");
		var memberData=objectData.cloneNode(true);
		topicNode.appendChild(memberData);
	}
	return bResult;
}

//apply the transform and return the result
function transformDoc()
{
	if (!getXSL())
	{
		return gErrorMessage;
	}
	return gxmlDoc.transformNode(gxslDoc);
	//Could send the results to a series of transforms or through a non-xsl transform
}

//get the XSL code that is used to transform this document
//This implementation looks at the PI in the document, strips "_v2" from the path and loads the XSL
//This could easily grab the stylesheet(s) from tmap or from nodes in the document itself
function getXSL()
{
	var piNode=gxmlDoc.childNodes(1);
	var piText=piNode.text
	var piAsDomNode="&lt;pinode " + piText + "/&gt;"
	var piDoc=new ActiveXObject("Microsoft.XMLDOM"); 
	piDoc.loadXML(piAsDomNode);
	var parseErr=piDoc.parseError;
	if (parseErr.errorCode != 0)
	{
		gErrorMessage= "Error loading XSL PI node " + parseErr.reason + " " + parseErr.line + " " + parseErr.srcText;
		bResult=false;
		return bResult;
	}
	piPath=piDoc.documentElement.getAttribute("href");
	piPath=piPath.substring(0, piPath.lastIndexOf("/")) + "/" + "&var_xslpath;";
	//piPath=piPath.replace("_v2","");

	gxslDoc.validateOnParse=false
	gxslDoc.async=false
	gxslDoc.resolveExternals=true
	gxslDoc.load(piPath);
	var parseErr=gxslDoc.parseError
	if (parseErr.errorCode != 0)
	{
		gErrorMessage= piPath + " " + parseErr.reason + " " + parseErr.line + " " + parseErr.srcText;
		bResult=false;
		return bResult;
	}
	return true;
}

